<?php

/**
 * @file
 * Contains \Drupal\email_octopus\Form\OctopusListForm.
 */

namespace Drupal\email_octopus\Form;

use Drupal\Core\Form\FormBase;
use GuzzleHttp\ClientInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactory;

/**
 * URl for API list.
 */
define("URL", "https://emailoctopus.com/api/1.5/lists?api_key=");

class OctopusListForm extends FormBase implements ContainerInjectionInterface {

  /**
   * {@inheritdoc}
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  protected $loggerFactory;

  function __construct(ClientInterface $http_client, LoggerChannelFactory $loggerFactory) {
    $this->httpClient = $http_client;
    $this->loggerFactory = $loggerFactory;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_client'), $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}.
   */
  public function getFormId() {
    return 'octopus_list_form';
  }

  /**
   * {@inheritdoc}.
   */
  public function buildForm(array $form, FormStateInterface $form_state, $values = NULL) {

    $form['table_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'table-wrapper'],
    ];
    $form['table_wrapper']['listid'] = [
      '#type' => 'select',
      '#title' => $this->t('Lists'),
      '#options' => $this->apiList(),
      '#required' => TRUE,
    ];
    $form['table_wrapper']['status'] = [
      '#type' => 'select',
      '#title' => $this->t('Subscribe/Unsubscribe'),
      '#options' => ['subscribed' => 'Subscribed',
        'unsubscribed' => 'Unsubscribed'
      ],
    ];
    $form['table_wrapper']['actions'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#ajax' => [
        'callback' => '::setMessage',
        'wrapper' => 'table-wrapper',
        'method' => 'replace',
        'effect' => 'fade',
      ],
    ];
    $header = [
      'email' => $this->t('Email'),
      'status' => $this->t('Status'),
    ];
    $form['table_wrapper']['value'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $values,
      '#empty' => $this->t('No list selected'),
    ];
    return $form;
  }

  /**
   * Returns wrapper to form.
   */
  public function setMessage(array $form, FormStateInterface $form_state) {
    return $form['table_wrapper'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $header = [
      'email' => $this->t('Email'),
      'status' => $this->t('Status'),
    ];
    $form['table_wrapper']['value'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $this->data($form_state->getValue('listid'), $form_state->getValue('status')),
      '#empty' => $this->t('No users found.'),
    ];
  }

  /*
   * Gets lists created at Email Octopus.
   */

  public function apiList() {
    $client = $this->httpClient;
    $key = $this->config('octopus.adminsettings')->get('api_key');
    if (!empty($key)) {
      $options = [
        'timeout' => 300,
        'headers' => ['Content-Type' => 'application/json'],
      ];
      $response = $client->request('GET', URL . $key, $options);
      $json = json_decode($response->getBody()->getContents(), TRUE);
      try {
        if (empty($json['data'])) {
          $message = "No data from API";
          $this->loggerFactory->get('Email Octopus')->warning($message);
          return [];
        }
        else {
          $group = [];
          foreach ($json['data'] as $option) {
            if ($option['id'] != NULL || $option['name'] != NULL) {
              $group[$option['id']] = $option['name'];
            }
          }
          return $group;
        }
      }
      catch (\Exception $e) {
        $this->loggerFactory->get('Email Octopus')->warning($e);
        throw $e;
      }
    }
    else {
      return [];
    }
  }

  /**
   * Return data from API according to List-Id and Status.
   *
   * Gets List-Id $listid and Status $status
   */
  public function data($listid, $status) {
    $data = [];
    $client = $this->httpClient;
    $key = $this->config('octopus.adminsettings')->get('api_key');
    $options = [
      'timeout' => 300,
      'headers' => ['Content-Type' => 'applifcation/json'],
    ];
    if ($listid != NULL) {
      $contacts = $client->request('GET', 'https://emailoctopus.com/api/1.5/lists/' . $listid . '/contacts/' . $status . '?api_key=' . $key, $options);
      $contactsJson = json_decode($contacts->getBody()->getContents(), TRUE);
      foreach ($contactsJson['data'] as $value) {
        $data[] = ['email' => $value['email_address'], 'status' => $value['status']];
      }
      return $data;
    }
  }

}
