<?php

/**
 * @file
 * Contains \Drupal\email_octopus\Plugin\Block\BookNowFormBlock.
 */

namespace Drupal\email_octopus\Plugin\Block;

use GuzzleHttp\ClientInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Logger\LoggerChannelFactory;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * URl for API list.
 */
define("URL", "https://emailoctopus.com/api/1.5/lists?api_key=");

/**
 * Provides a 'Octopus Subscribe Form' block.
 *
 * @Block(
 *   id = "subscribe_form_block",
 *   admin_label = @Translation("Octopus Subscribe Form"),
 *   category = @Translation("Custom Block")
 * )
 */
class SubscribeFormBlock extends BlockBase implements \Drupal\Core\Plugin\ContainerFactoryPluginInterface {

  /**
   *
   * @var Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   *
   * @var GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   *
   * @var Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configfactory;

  /**
   *
   * @var Drupal\Core\Logger\LoggerChannelFactory
   */
  protected $loggerFactory;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Renderer $renderer, ClientInterface $http_client, ConfigFactoryInterface $config_factory, LoggerChannelFactory $loggerFactory, FormBuilderInterface $formBuilder) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->renderer = $renderer;
    $this->httpClient = $http_client;
    $this->configfactory = $config_factory;
    $this->loggerFactory = $loggerFactory;
    $this->formBuilder = $formBuilder;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration, $plugin_id, $plugin_definition, $container->get('renderer'), $container->get('http_client'), $container->get('config.factory'), $container->get('logger.factory'), $container->get('form_builder')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();
    $form['api_list'] = [
      '#type' => 'select',
      '#title' => $this->t('Lists'),
      '#options' => $this->apiList(),
      '#default_value' => isset($config['api_list']) ? $config['api_list'] : '',
      '#required' => TRUE,
    ];
    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Block Title'),
      '#description' => $this->t('Heading for block'),
      '#default_value' => isset($config['title']) ? $config['title'] : '',
      '#required' => TRUE,
    ];
    $form['body'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Block Body'),
      '#description' => $this->t('Content'),
      '#default_value' => isset($config['body']) ? $config['body'] : '',
      '#required' => TRUE,
    ];
    $form['message'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Thankyou Message'),
      '#description' => $this->t('Message for user on successful registration.'),
      '#default_value' => isset($config['message']) ? $config['message'] : '',
      '#required' => TRUE,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->setConfigurationValue('api_list', $form_state->getValue('api_list'));
    $this->setConfigurationValue('title', $form_state->getValue('title'));
    $this->setConfigurationValue('body', $form_state->getValue('body'));
    $this->setConfigurationValue('message', $form_state->getValue('message'));
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $apilist = $config['api_list'];
    $title = $config['title'];
    $body = $config['body'];
    $message = $config['message'];
    $build = [];
    $form = $this->formBuilder->getForm('\Drupal\email_octopus\Form\OctopusSubscribeForm', $apilist, $title, $body, $message);
    $build['#markup'] = $this->renderer->render($form);
    $build['#cache'] = ['max-age' => 0];
    return $build;
  }

  /*
   * This functions gets lists created at Email Octopus.
   */

  public function apiList() {
    $client = $this->httpClient;
    $key = $this->configfactory->getEditable('octopus.adminsettings')->get('api_key');
    if ($key != NULL) {
      $options = [
        'timeout' => 300,
        'headers' => ['Content-Type' => 'application/json'],
      ];
      $response = $client->request('GET', URL . $key, $options);
      $json = json_decode($response->getBody()->getContents(), TRUE);
      try {
        if (empty($json['data'])) {
          $message = "No data from API";
          $this->loggerFactory->get('Email Octopus')->warning($message);
        }
        else {
          $group = [];
          foreach ($json['data'] as $option) {
            if ($option['id'] != NULL || $option['name'] != NULL) {
              $group[$option['id']] = $option['name'];
            }
          }
          return $group;
        }
      }
      catch (\Exception $e) {
        $this->loggerFactory->get('Email Octopus')->warning($e);
        throw $e;
      }
    }
    else {
      return [];
    }
  }

}
